#!/usr/bin/python

import re

mapping = {'\\`A': u'\xc0',
'\\`E': u'\xc8',
'\\`I': u'\xcc',
'\\`O': u'\xd2',
'\\`U': u'\xd9',
'\\`a': u'\xe0',
'\\`e': u'\xe8',
'\\`i': u'\xec',
'\\`o': u'\xf2',
'\\`u': u'\xf9',
"\\'A": u'\xc1',
"\\'E": u'\xc9',
"\\'I": u'\xcd',
"\\'O": u'\xd3',
"\\'U": u'\xda',
"\\'Y": u'\xdd',
"\\'a": u'\xe1',
"\\'e": u'\xe9',
"\\'i": u'\xed',
"\\'o": u'\xf3',
"\\'u": u'\xfa',
"\\'y": u'\xfd',
"\\'S": u'\u015a',
"\\'Z": u'\u0179',
"\\'s": u'\u015b',
"\\'z": u'\u017a',
"\\'R": u'\u0154',
"\\'L": u'\u0139',
"\\'C": u'\u0106',
"\\'N": u'\u0143',
"\\'r": u'\u0155',
"\\'l": u'\u013a',
"\\'c": u'\u0107',
"\\'n": u'\u0144',
'\\^A': u'\xc2',
'\\^E': u'\xca',
'\\^I': u'\xce',
'\\^O': u'\xd4',
'\\^U': u'\xdb',
'\\^a': u'\xe2',
'\\^e': u'\xea',
'\\^i': u'\xee',
'\\^o': u'\xf4',
'\\^u': u'\xfb',
'\\^H': u'\u0124',
'\\^J': u'\u0134',
'\\^h': u'\u0125',
'\\^j': u'\u0135',
'\\^C': u'\u0108',
'\\^G': u'\u011c',
'\\^S': u'\u015c',
'\\^c': u'\u0109',
'\\^g': u'\u011d',
'\\^s': u'\u015d',
'\\,C': u'\xc7',
'\\,c': u'\xe7',
'\\,S': u'\u015e',
'\\,s': u'\u015f',
'\\,T': u'\u0162',
'\\,t': u'\u0163',
'\\,R': u'\u0156',
'\\,L': u'\u013b',
'\\,G': u'\u0122',
'\\,r': u'\u0157',
'\\,l': u'\u013c',
'\\,g': u'\u0123',
'\\,N': u'\u0145',
'\\,K': u'\u0136',
'\\,n': u'\u0146',
'\\,k': u'\u0137',
'\\"A': u'\xc4',
'\\"E': u'\xcb',
'\\"I': u'\xcf',
'\\"O': u'\xd6',
'\\"U': u'\xdc',
'\\"a': u'\xe4',
'\\"e': u'\xeb',
'\\"i': u'\xef',
'\\"o': u'\xf6',
'\\"u': u'\xfc',
'\\"y': u'\xff',
'\\~A': u'\xc3',
'\\~N': u'\xd1',
'\\~O': u'\xd5',
'\\~a': u'\xe3',
'\\~n': u'\xf1',
'\\~o': u'\xf5',
'\\~I': u'\u0128',
'\\~i': u'\u0129',
'\\~U': u'\u0168',
'\\~u': u'\u0169',
'\\oA': u'\xc5',           
'\\oa': u'\xe5',
'\\oU': u'\u016e',
'\\ou': u'\u016f',
'\\=E': u'\u0112',
'\\=e': u'\u0113',
'\\=A': u'\u0100',
'\\=I': u'\u012a',
'\\=O': u'\u014c',
'\\=U': u'\u016a',
'\\=a': u'\u0101',
'\\=i': u'\u012b',
'\\=o': u'\u014d',
'\\=u': u'\u016b',
'\\=D': u'\u0110',
'\\=d': u'\u0111',
'\\=H': u'\u0126',
'\\=h': u'\u0127',
'\\=T': u'\u0166',
'\\=t': u'\u0167',
'\\/O': u'\xd8',
'\\/o': u'\xf8',
'\\/D': u'\u0110',
'\\/d': u'\u0111',
'\\/L': u'\u0141',
'\\/l': u'\u0142',
'"\\;A"': u'\u0104',
'"\\;a"': u'\u0105',
'"\\;E"': u'\u0118',
'"\\;e"': u'\u0119',
'"\\;I"': u'\u012e',
'"\\;U"': u'\u0172',
'"\\;i"': u'\u012f',
'"\\;u"': u'\u0173',
'\\vL': u'\u013d',
'\\vS': u'\u0160',
'\\vT': u'\u0164',
'\\vZ': u'\u017d',
'\\vl': u'\u013e',
'\\vs': u'\u0161',
'\\vt': u'\u0165',
'\\vz': u'\u017e',
'\\vC': u'\u010c',
'\\vE': u'\u011a',
'\\vD': u'\u010e',
'\\vN': u'\u0147',
'\\vR': u'\u0158',
'\\vc': u'\u010d',
'\\ve': u'\u011b',
'\\vd': u'\u010f',
'\\vn': u'\u0148',
'\\vr': u'\u0159',
'\\uA': u'\u0102',
'\\ua': u'\u0103',
'\\uG': u'\u011e',
'\\ug': u'\u011f',
'\\uU': u'\u016c',
'\\uu': u'\u016d',
'\\:O': u'\u0150',
'\\:U': u'\u0170',
'\\:o': u'\u0151',
'\\:u': u'\u0171',
'\\.Z': u'\u017b',
'\\.z': u'\u017c',
'\\.I': u'\u0130',
'\\.i': u'\u0131',
'\\.C': u'\u010a',
'\\.G': u'\u0120',
'\\.c': u'\u010b',
'\\.g': u'\u0121',
'\\.E': u'\u0116',
'\\.e': u'\u0117',
'(C)': u'\xa9',

'\\NS': u'\xa0',
'\\!!': u'\xa1',
'\\Ct': u'\xa2',
'\\Pd': u'\xa3',
'\\Cu': u'\xa4',
'\\Ye': u'\xa5',
'\\BB': u'\xa6',
'\\SE': u'\xa7',
"\\\':": u'\xa8',
'\\Co': u'\xa9',
'\\-a': u'\xaa',
'\\<<': u'\xab',
'\\NO': u'\xac',
'\\--': u'\xad',
'\\Rg': u'\xae',
"\\\'-": u'\xaf',
'\\DG': u'\xb0',
'\\+-': u'\xb1',
'\\2S': u'\xb2',
'\\3S': u'\xb3',
"\\\'\'": u'\xb4',
'\\My': u'\xb5',
'\\PI': u'\xb6',
'\\.M': u'\xb7',
"\\\',": u'\xb8',
'\\1S': u'\xb9',
'\\-o': u'\xba',
'\\>>': u'\xbb',
'\\14': u'\xbc',
'\\12': u'\xbd',
'\\34': u'\xbe',
'\\?I': u'\xbf',
'\\A!': u'\xc0',
"\\A\'": u'\xc1',
'\\A>': u'\xc2',
'\\A?': u'\xc3',
'\\A:': u'\xc4',
'\\AA': u'\xc5',
'\\AE': u'\xc6',
'\\C,': u'\xc7',
'\\E!': u'\xc8',
"\\E\'": u'\xc9',
'\\E>': u'\xca',
'\\E:': u'\xcb',
'\\I!': u'\xcc',
"\\I\'": u'\xcd',
'\\I>': u'\xce',
'\\I:': u'\xcf',
'\\D-': u'\xd0',
'\\N?': u'\xd1',
'\\O!': u'\xd2',
"\\O\'": u'\xd3',
'\\O>': u'\xd4',
'\\O?': u'\xd5',
'\\O:': u'\xd6',
'\\*X': u'\xd7',
'\\O/': u'\xd8',
'\\U!': u'\xd9',
"\\U\'": u'\xda',
'\\U>': u'\xdb',
'\\U:': u'\xdc',
"\\Y\'": u'\xdd',
'\\TH': u'\xde',
'\\ss': u'\xdf',
'\\a!': u'\xe0',
"\\a\'": u'\xe1',
'\\a>': u'\xe2',
'\\a?': u'\xe3',
'\\a:': u'\xe4',
'\\aa': u'\xe5',
'\\ae': u'\xe6',
'\\c,': u'\xe7',
'\\e!': u'\xe8',
"\\e\'": u'\xe9',
'\\e>': u'\xea',
'\\e:': u'\xeb',
'\\i!': u'\xec',
"\\i\'": u'\xed',
'\\i>': u'\xee',
'\\i:': u'\xef',
'\\d-': u'\xf0',
'\\n?': u'\xf1',
'\\o!': u'\xf2',
"\\o\'": u'\xf3',
'\\o>': u'\xf4',
'\\o?': u'\xf5',
'\\o:': u'\xf6',
'\\-:': u'\xf7',
'\\o/': u'\xf8',
'\\u!': u'\xf9',
"\\u\'": u'\xfa',
'\\u>': u'\xfb',
'\\u:': u'\xfc',
"\\y\'": u'\xfd',
'\\th': u'\xfe',
'\\y:': u'\xff',
'"\\A;"': u'\u0104',
"\\\'(": u'\u02d8',
'\\L/': u'\u0141',
'\\L<': u'\u013d',
"\\S\'": u'\u015a',
'\\S<': u'\u0160',
'\\S,': u'\u015e',
'\\T<': u'\u0164',
"\\Z\'": u'\u0179',
'\\Z<': u'\u017d',
'\\Z.': u'\u017b',
'"\\a;"': u'\u0105',
"\\\'(": u'\u02d8',
'\\l/': u'\u0142',
'\\l<': u'\u013e',
"\\s\'": u'\u015b',
"\\\'<": u'\u02c7',
'\\s<': u'\u0161',
'\\s,': u'\u015f',
'\\t<': u'\u0165',
"\\z\'": u'\u017a',
'\\\'\"': u'\u02dd',
'\\z<': u'\u017e',
'\\z.': u'\u017c',
"\\R\'": u'\u0154',
'\\A(': u'\u0102',
"\\L\'": u'\u0139',
"\\C\'": u'\u0106',
'\\C<': u'\u010c',
'"\\E;"': u'\u0118',
'\\E<': u'\u011a',
'\\D<': u'\u010e',
'\\D/': u'\u0110',
"\\N\'": u'\u0143',
'\\N<': u'\u0147',
'\\O\"': u'\u0150',
'\\R<': u'\u0158',
'\\U0': u'\u016e',
'\\U\"': u'\u0170',
'\\T,': u'\u0162',
"\\r\'": u'\u0155',
'\\a(': u'\u0103',
"\\l\'": u'\u013a',
"\\c\'": u'\u0107',
'\\c<': u'\u010d',
'"\\e;"': u'\u0119',
'\\e<': u'\u011b',
'\\d<': u'\u010f',
'\\d/': u'\u0111',
"\\n\'": u'\u0144',
'\\n<': u'\u0148',
'\\o\"': u'\u0151',
'\\r<': u'\u0159',
'\\u0': u'\u016f',
'\\u\"': u'\u0171',
'\\t,': u'\u0163',
"\\\'.": u'\u02d9',
'\\H/': u'\u0126',
'\\H>': u'\u0124',
'\\I.': u'\u0130',
'\\G(': u'\u011e',
'\\J>': u'\u0134',
'\\h/': u'\u0127',
'\\h>': u'\u0125',
'\\i.': u'\u0131',
'\\g(': u'\u011f',
'\\j>': u'\u0135',
'\\C.': u'\u010a',
'\\C>': u'\u0108',
'\\G.': u'\u0120',
'\\G>': u'\u011c',
'\\U(': u'\u016c',
'\\S>': u'\u015c',
'\\c.': u'\u010b',
'\\c>': u'\u0109',
'\\g.': u'\u0121',
'\\g>': u'\u011d',
'\\u(': u'\u016d',
'\\s>': u'\u015d',
'\\kk': u'\u0138',
'\\R,': u'\u0156',
'\\I?': u'\u0128',
'\\L,': u'\u013b',
'\\E-': u'\u0112',
'\\G,': u'\u0122',
'\\T/': u'\u0166',
'\\r,': u'\u0157',
'\\i?': u'\u0129',
'\\l,': u'\u013c',
'\\e-': u'\u0113',
'\\g,': u'\u0123',
'\\t/': u'\u0167',
'\\NG': u'\u014a',
'\\ng': u'\u014b',
'\\A-': u'\u0100',
'"\\I;"': u'\u012e',
'\\E.': u'\u0116',
'\\I-': u'\u012a',
'\\N,': u'\u0145',
'\\O-': u'\u014c',
'\\K,': u'\u0136',
'"\\U;"': u'\u0172',
'\\U?': u'\u0168',
'\\U-': u'\u016a',
'\\a-': u'\u0101',
'"\\i;"': u'\u012f',
'\\e.': u'\u0117',
'\\i-': u'\u012b',
'\\n,': u'\u0146',
'\\o-': u'\u014d',
'\\k,': u'\u0137',
'"\\u;"': u'\u0173',
'\\u?': u'\u0169',
'\\u-': u'\u016b',           

r'{\aa}': u'\xe5',
r'{\aA}': u'\xc5',}

reverse_mapping = dict((b, a) for (a, b) in mapping.items())
encoded_char_re = re.compile('|'.join(re.escape(k) for k in mapping))

def decode_abc(abc_code): return encoded_char_re.sub(lambda m: mapping[m.group(0)], abc_code)
def encode_abc(abc_code): return ''.join(reverse_mapping.get(c, c) for c in abc_code)
	