#gcgen.py

'''
The notes in the expanded gchord are placed in the
voice chord track note by note as the body of the
abc file is scanned. This allows handling of incomplete
measures and chord changes occurring inside a bar.

gchord_generator is called each time a note in the
body is scanned. Anytime the gchord string (eg fczfcz)
is changed, setup_gchord() is called.

At the end of each bar, we call process_barline
to ensure that the corresponding bar in the gchord
track matches the length of the bar in the abc body.

gchordindex keeps track of the index in the
gchord string.

gchordunit is a time unit for each gchord string
element.

gchordaccumulator and bar_accumulator accumulate
the time units.

'''


import gcstring
import meter

gchordaccumulator = 0
gchordindex = 0
gchordunit = 0
bar_accumulator = 0


def process_barline(token):
    global gchordindex, gchordaccumulator
    global bar_accumulator
    global expandedchords_for_line
    if gchordaccumulator < bar_accumulator:
        adjust = int((bar_accumulator - gchordaccumulator)/192)
        adjustelem = 'z'+str(adjust)
        expandedchords_for_line = expandedchords_for_line + ' '+ adjustelem

    bar_accumulator = 0
    gchordindex = 0
    gchordaccumulator = 0
    expandedchords_for_line = expandedchords_for_line + ' '+ token


def setup_gchord_generator():
    global gchordaccumulator,gchordindex
    global gchordunit
    if gcstring.gc_string_length != 0:
        gchordunit = meter.barunits/gcstring.gc_string_length
        gchordindex = 0
    else:
        print('no default gstring for current meter please include a %%MIDI gchord command in your tune')
        quit()

expandedchords_for_line = ''

def gchord_generator():
    global gchordindex
    global expandedchords_for_line
    global gchordaccumulator
    if not gcstring.gchordoutput: return
    if bar_accumulator < gchordaccumulator: return
    while gchordaccumulator < bar_accumulator:
        if gchordindex >= len(gcstring.gchordoutput): return
        gchord_output_element = gcstring.gchordoutput[gchordindex]
        gchordelem = gcstring.gcstrlist[gchordindex]
        if len(gchordelem)>1:
            n = gchordelem[1]
        else: n = 1
        gchordaccumulator = gchordaccumulator + gchordunit*int(n)
        expandedchords_for_line = expandedchords_for_line + ' '+ gchord_output_element+ ' '
        gchordindex = gchordindex+1

def clear_expandedchords_for_line():
    global expandedchords_for_line
    expandedchords_for_line = ''

def return_gchord_unitlength():
    if gchordunit > 767:
        return 'L:1/2'
    if gchordunit > 383:
        return 'L:1/4'
    if gchordunit > 191:
        return 'L:1/8'
    if gchordunit > 95:
        return 'L:1/16'
    return 'L:1/32'
