#gchord.py

'''
The guitar chord indications in the body of the abc file
need to be translated in actual notes. Guitar chords
can be quite complex and contain inversions (eg GM7/D).
Also the notes need to be translated to the current
key signature to avoid a proliferation of accidentals.

The results are stored in the lists midichord and
gchordnotes. bassnote is the note corresponding to
f in the gchord string.
'''


import re
import copy
import key

#chord descriptors; lists offsets of chord notes in semitones
#relative to the bass note.
chordnames = {'Maj':  [0,4,7],
              'm':    [0,3,7],
              '7':    [0,4,7,10],
              'm7':   [0,3,7,10],
              'maj7': [0,4,7,11],
              'M7':   [0,4,7,11],
              '6':    [0,4,7,9],
              'm6':   [0,3,7,9],
              'aug':  [0,4,8],
              'plus': [0,4,8],
              'aug7': [0,4,8,10],
              'dim':  [0,3,6],
              'dim7': [0,3,6,9],
              '9':    [0,4,7,10,2],
              'm9':   [0,3,7,10,2],
              'maj9': [0,4,7,11,2],
              'M9':   [0,4,7,11,2],
              '11':   [0,4,7,10,2,5],
              'dim9': [0,4,7,10,13],
              'sus':  [0,5,7],
              '7sus4':[0,5,7,10],
              '7sus9':[0,2,7,10],
              '5':    [0,7]}


inverpat =  re.compile('(.+)/(.+)')
gcpat    =  re.compile('([A-G]|[a-g]|)(#+|b?)(.*)')
keypat    =  re.compile('([A-G]|[a-g]|)(#|b?)')
notescale = (0, 2, 4, 5, 7, 9, 11)
notekey = 'CDEFGABcdefgab'

def initialize_gchord():
    global gchordnotes
    global bassnote
    gchordnotes = []
    bassnote = ''

def expandgchord(gchord):
# expandgchord {gchord}
# The function interprets a gchord (enclosed by a pair of double quotes) in
# the music body and returns the equivalent notes forming the bass/chordal
# accompaniment in the variables bassnote and gchordnotes.
# Notes are converted to numeric scale C 0, C# 1, D 2 etc called
# MIDI units. We compute the chordal notes in MIDI units and then
# transform back to the musical scale.
    global gchordnotes, bassnote
    #print 'expandgchord(',gchord,')'
    inversion = -1
    m1 = inverpat.match(gchord)
    if m1:
# if it is an inversion identify the note
        gchord =  m1.groups()[0]
        invert =  m1.groups()[1]
        m2 = keypat.match(invert)
        note = m2.groups()[0]
        accidental = m2.groups()[1]
        n = notekey.find(invert[0])
        n = n % 7
        midiinversion = notescale[n]
        if accidental=='#':
            midiinversion = midiinversion+1
        if accidental=='b':
            midiinversion = midiinversion-1

# now splitup the gchord
    m1 = gcpat.match(gchord)
    if not m1:
        print('cannot understand',gchord)
        return
    bass = m1.groups()[0]
    accidental = m1.groups()[1]
    type = m1.groups()[2]
    if bass=='none':
        bass = C
    n = notekey.find(bass[0])
    n = n%7
    midibass = notescale[n]
    if accidental == '#': midibass=midibass+1
    if accidental == 'b': midibass=midibass-1
    if type=='': type='Maj'

# add midibass to the notes in chordnames[type]
    midichord = copy.copy(chordnames[type])
    i = 0
    while i<len(midichord):
        midichord[i]= midichord[i] +midibass
        i = i+1

# if inversion find inversion note in chord
    if inversion != -1:
        i = 0
        while i <len(midichord):
            if midichord[i]==inversion: inchord=i
            i=i+1
#  applyinversion
        i = 0
        while i <len(midichord):
            if i<inchord: midichord[i]=midichord[i]+12
            i=i+1

    i = 0
    while i<len(midichord):
        midichord[i] = midichord[i]+48
        i = i+1


    i=0
    bassnote = key.midi2key(midibass+36)
    gchordnotes=[]
    while i<len(midichord):
        gchordnotes.append(key.midi2key(midichord[i]))
        i=i+1
