#meter.py
'''
This module processes the L: and M: commands in the
abc tune and sets the state variables
noteunits -- the length of a quarter note
barunits -- the length of a bar or measure

The smallest note is 1/128; to handle triplets we also
introduce a factor of 3. So each quarter note will be
represented by 384 time units. The length of a bar
is computed in these time units (barunits). The length
of an abc note, determined by L:1/y determines the
note length in these units (noteunits).

For familiar time signatures, we select a default gchord
string in case one is not given.
'''

import re
import gcstring

default_gchord = {'2/2': 'fzczfzcz', '2/4': 'fzczfzcz',
                  '4/4': 'fzczfzcz', '3/4': 'fzczcz',
                  '6/8': 'fzcfzc',  '12/8': 'fzcfzcfzcfzc'}

barunits  = 0
noteunits = 0
fracpat  = re.compile('(\d+)/(\d+)')

def setup_unitlength(unit):
    global noteunits
    m = fracpat.match(unit)
    num = int(m.group(1))
    den = int(m.group(2))
    noteunits = 384*4*num/den

def setup_meter(meter):
    global barunits
    global noteunits
    m = fracpat.match(meter)
    num = int(m.group(1))
    den = int(m.group(2))
    barunits = 4*384*num/den
    if meter in default_gchord:
        gc = default_gchord[meter]
        gcstring.scangchordstring(gc)
    # in case L: is missing
    if num == 4:
        noteunits = 384
    else:
        noteunits = 192

